/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.write.request;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.CancelTransactionException;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.TransactionSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

/**
 * Checks that asynchronous read requests work correctly with canceled writes
 * 
 * @author Antti Villberg
 */
public class WriteCancelTest2 extends ExistingDatabaseTest {

    private static final int SIZE = 10000;

    Resource[] written = new Resource[SIZE];

	@Test
    public void testWriteCancel2() throws DatabaseException{

        // First create a set of resources for testing
        try {

            getSession().syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    if (DEBUG)
                        System.err.println("a");
                    for(int i=0;i<SIZE;i++) written[i] = graph.newResource();
                    if (DEBUG)
                        System.err.println("a2");
                }
            });
            
        } catch (DatabaseException e) {
            if (DEBUG)
                e.printStackTrace();
        }

        // Make a decent amount of modifications and cancel them
        getSession().asyncRequest(new WriteRequest() {
            @Override
            public void perform(WriteGraph graph) throws DatabaseException {
                Layer0 L0 = Layer0.getInstance(graph);
                if (DEBUG)
                    System.err.println("b");
                for(int i=0;i<SIZE;i++) graph.claim(written[i], L0.IsComposedOf, graph.newResource());
                if (DEBUG)
                    System.err.println("b2");
                throw new CancelTransactionException("Intentional write request cancellation");
            }
        });
        
        // Ensure that the previously written data does not exist.
        getSession().asyncRequest(new ReadRequest() {
            @Override
            public void run(ReadGraph graph) throws DatabaseException {

                Layer0 L0 = Layer0.getInstance(graph);
                if (DEBUG)
                    System.err.println("c");
                for(int i=0;i<SIZE;i++)
                    assertTrue(!graph.hasStatement(written[i], L0.IsComposedOf));
                if (DEBUG)
                    System.err.println("c2");

            }
        });

        // Do not let the test die before asynchronous requests have finished
        TransactionSupport support = getSession().getService(TransactionSupport.class);
        support.waitCompletion();
        
    }

}
