/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.write.request;

import java.util.ArrayList;
import java.util.UUID;

import org.junit.Test;
import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteOnlyGraph;
import org.simantics.db.common.request.WriteOnlyRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

public class WriteOnlyReadTest extends ExistingDatabaseTest {

    final static int ITEM_COUNT = 2;
    
	@Test
    public void testWriteException() throws Exception {

        final Session session = getSession();
        final Layer0 b = Layer0.getInstance(session);

        final ArrayList<Resource> items = new ArrayList<Resource>();
        
        session.syncRequest(new WriteOnlyRequest() {

            @Override
            public void perform(WriteOnlyGraph graph) throws DatabaseException {
                
                graph.flushCluster();
                for(int i=0;i<ITEM_COUNT;i++) {
                    Resource item = graph.newResource();
                    graph.claim(item, b.InstanceOf, null, b.String);
                    graph.claimValue(item, UUID.randomUUID().toString(), Bindings.STRING);
                    //graph.claim(g.getRootLibrary(), b.ConsistsOf, b.PartOf, item);
                    items.add(item);
                }

            }

        });

        session.syncRequest(new TestReadRequest() {

            @Override
            public void run(ReadGraph graph) throws DatabaseException {
                //Layer0 b = Layer0.getInstance(g);
                for(Resource r : items) {
                    String value = graph.getPossibleValue(r, Bindings.STRING);
                    if (DEBUG)
                        System.out.println("value=" + value);
                }

            }

        });
        
    }
}
