/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.performance.read;

import java.util.ArrayList;
import java.util.Collection;
import java.util.UUID;

import org.simantics.databoard.Bindings;
import org.simantics.datatypes.utils.LogUtils;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.VirtualGraph;
import org.simantics.db.WriteGraph;
import org.simantics.db.WriteOnlyGraph;
import org.simantics.db.common.request.WriteOnlyResultRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.common.request.WriteResultRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.VirtualGraphSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.utils.datastructures.Pair;


public class LogTest2 extends ExistingDatabaseTest {

	public static final int NODE_SIZE = 512;
	public static final int TEN = 10;
	public static final int HUNDRED = 100;
	public static final int THOUSAND = 1000;
	public static final int MILLION = 1000000;
	
	public static final int KEY_SIZE = THOUSAND;
	public static final int ROUNDS = TEN * HUNDRED;
	public static final int BATCHES = 5;
	
	public void test() throws Exception {
		
		Session session = getSession();
		
		VirtualGraphSupport vgss = session.getService(VirtualGraphSupport.class);
		VirtualGraph vg = vgss.getMemoryPersistent(UUID.randomUUID().toString());

		final Resource log = session.sync(new WriteResultRequest<Resource>(vg) {

			@Override
			public Resource perform(WriteGraph graph) throws DatabaseException {
				
				LogUtils lu = new LogUtils(graph);
				return lu.create(graph, NODE_SIZE, 0);
				
			}

		});		
		
		for(int i=0;i<BATCHES;i++) {

			long start = System.nanoTime();

			for(int j=0;j<ROUNDS;j++) {

				final Collection<Pair<Integer, Resource>> values = session.sync(new WriteOnlyResultRequest<Collection<Pair<Integer, Resource>>>(vg) {

					@Override
					public Collection<Pair<Integer, Resource>> perform(WriteOnlyGraph graph) throws DatabaseException {

						Collection<Pair<Integer, Resource>> values = new ArrayList<Pair<Integer, Resource>>();
						for(int i=0;i<KEY_SIZE;i++) {
							Resource value = graph.newResource();
							graph.claim(value, L0.InstanceOf, null, L0.String);
							graph.claimValue(value, "Value" + i, Bindings.STRING);
							values.add(Pair.make(i, value));
						}
						return values;

					}

				});


				session.sync(new WriteRequest(vg) {

					@Override
					public void perform(WriteGraph graph) throws DatabaseException {

						LogUtils lu = new LogUtils(graph);
						lu.insertAll(graph, log, values);

					}

				});

			}
			
			long duration = System.nanoTime()-start;
			System.err.println("A batch of " + KEY_SIZE*ROUNDS + " values at " + 1e-9*duration + "s.");
			start = System.nanoTime();

		}

	}

}
