package org.simantics.db.tests.regression.bugs;

import java.util.UUID;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.WriteOnlyGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteOnlyRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.ClusterControl;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.db.testing.common.TestBase;
import org.simantics.layer0.Layer0;

public class SimanticsBug1659Test3 extends ExistingDatabaseTest {
    static int LOOP_COUNT = 10;
    static int CLUSTER_COUNT = 20;
    static int RESOURCE_COUNT = 10000;
    static boolean DEBUG = false;
    Session session;
    Resource testRoot;
    ClusterControl clusterControl;
    @Test
    public void testSimanticsBug1659_3()
    throws DatabaseException {
        session = getSession();
        clusterControl = getSession().getService(ClusterControl.class);
        session.syncRequest(new Init());
        session.syncRequest(new CreateWriteOnly());
        for (int i=0; i<LOOP_COUNT; ++i) {
            session.syncRequest(new Query());
        }
    }
    class Init extends WriteRequest {
        @Override
        public void perform(WriteGraph g) throws DatabaseException {
            Layer0 l0 = Layer0.getInstance(g);
            Resource rl = g.getResource(TestBase.ROOT_LIBRARY_URI);
            testRoot = g.newResource();
            g.claim(testRoot, l0.InstanceOf, l0.Library);          
            g.claim(rl, l0.ConsistsOf, testRoot);
        }
    }
    class CreateWriteOnly extends WriteOnlyRequest {
        @Override
        public void perform(WriteOnlyGraph g) throws DatabaseException {
            Layer0 b = Layer0.getInstance(getSession());
            for (int j=0; j<CLUSTER_COUNT; ++j) {
                Resource root = g.newResource();
                g.claim(testRoot, b.ConsistsOf, b.PartOf, root);
                g.flushCluster();
                for(int i=0; i<RESOURCE_COUNT; i++) {
                    Resource item = g.newResource();
                    g.claim(item, b.InstanceOf, null, b.String);
                    g.claimValue(item, UUID.randomUUID().toString());
                    g.claim(root, b.ConsistsOf, b.PartOf, item);
                }
            }
        }
    }
    class Query extends ReadRequest {
        @Override
        public void run(ReadGraph g) throws DatabaseException {
            Layer0 l0 = Layer0.getInstance(g);
            for (Resource r : g.getObjects(testRoot, l0.ConsistsOf)) {
                if (DEBUG)
                    System.out.println("Resource " + r);
                for (Resource rr : g.getObjects(r, l0.ConsistsOf)) {
                	if (!g.isInstanceOf(rr, l0.String))
                		fail("Resource " + rr + " is not instance of String.");
                	if (!g.isInstanceOf(rr, l0.Entity))
                		fail("Resource " + rr + " is not instance of Entity.");
                }
            }
        }
    }
}
